c**********************************************************************
c IO_INIT_FILTER.F
c**********************************************************************
c Read in filter parameters, precompute filters.
c
c Filter/decimation process was published in Quinn, T. R., et al.:
c Integration of Earth's orbit. AJ 101, p. 2287, 1991
c
c These subroutines and functions are used to obtain long term
c variations of orbital elements.
c
c Input:
c  infile	name of the filter parameters input file,
c		for structure of this file refer to example filter.in
c
c Output:
c  filter_A(),filter_B()	filters A,B
c  filter_M			dimension of above filters
c  filter_N			number of filter/decimation cycles
c  filter_D(filter_N)		decimation parameter for each cycle
c  filter_stat(filter_N)	char array, which filter to use in the cycle?
c  FBUFN			number of samples to store in memory
c  dtfilter			sampling period
c  outfile			name of binary output file
c
c Remarks: 
c Authors:  Miroslav Broz, miroslav.broz@usa.net
c Date:  6/12/1998 

	subroutine io_init_filter(infile,dtfilter,outfile)

	include 'swift.inc'
	include 'filter.inc'

c...    Input
	character*80 infile

c...    Output
	real*8 dtfilter
	character*80 outfile
c
c  parameters of filtration process are given in common block /filter/
c
      real*8 A(-FMAXM:FMAXM),B(-FMAXM:FMAXM),dtfilter_common
      integer filter_M,filter_N,filter_D(FMAXC),FBUFN
      character*10 filter_stat  ! have to be the last in /filter/

      common/filter/A,B,dtfilter_common,filter_M,filter_N,filter_D,
     &  FBUFN,filter_stat

c...   Internal
       integer i,ierr,AM,BM
	real*8 Ax0,Bx0,Abeta,Bbeta
c-----
c...  Executable code      

      write(*,*) 'Filter parameters file called ',infile
      call io_open(7,infile,'old','formatted',ierr)

      write(*,*) ' '
      read(7,*) filter_N
      read(7,10) filter_stat
10    format(a)
      read(7,*) (filter_D(i),i=1,filter_N)
      read(7,*) AM,Ax0,Abeta
      read(7,*) BM,Bx0,Bbeta
c
c  sampling period, output to outfile will be produced every
c  filter_D(1) * filter_D(2) * ... * filter_D(N) * dtfilter, see FBUFN
c
      read(7,*) dtfilter
c  give dtfilter both as a parameter and in common block 
      dtfilter_common = dtfilter
      read(7,10) outfile

      close(unit = 7)
      write(*,*) ' '

      call filter_create(A,AM,Ax0,Abeta)
      call filter_create(B,BM,Bx0,Bbeta)
      filter_M = AM		! expecting AM=BM
      FBUFN = 2*filter_M+1	! width of buffers

      return
      end    ! io_init_filter
c-----------------------------------------------------------------

c****************************************************************
      double precision function I0(x)
c****************************************************************
c
c  Modified Bessel function
c  from Numerical Recipes, p. 273, function bessi0
c  http://cfatab.harvard.edu/nr/bookf.html
c
      implicit none
      double precision x
      double precision ax,p1,p2,p3,p4,p5,p6,p7,
     &  q1,q2,q3,q4,q5,q6,q7,q8,q9,y
      save p1,p2,p3,p4,p5,p6,p7,q1,q2,q3,q4,q5,q6,q7,q8,q9
      data p1,p2,p3,p4,p5,p6,p7/1.0d0,3.5156229d0,3.0899424d0,
     &  1.2067492d0,0.2659732d0,0.360768d-1,0.45813d-2/
      data q1,q2,q3,q4,q5,q6,q7,q8,q9/0.39894228d0,0.1328592d-1,
     &  0.225319d-1,-0.157565d-1,0.916281d-2,-0.2057706,
     &  0.2635537d-1,-0.1647633d-1,0.392377d-1/

      if (abs(x).lt.3.75) then
        y=(x/3.75)**2
        I0=p1+y*(p2+y*(p3+y*(p4+y*(p5+y*(p6+y*p7)))))
      else
        ax=abs(x)
        y=3.75/ax
        I0=(exp(ax)/sqrt(ax))*(q1+y*(q2+y*(q3+y*(q4
     &     +y*(q5+y*(q6+y*(q7+y*(q8+y*q9))))))))
      endif
      return
      end

c****************************************************************
      subroutine filter_create(d,Mm,x0,beta)
c****************************************************************
c
c  create filter
c  d(Mm)       filter
c  Mm,x0,beta  parameters of the filter
c
      implicit none
      include 'filter.inc'
      real*8 pi
      parameter (pi = 3.14159265358979d0)

c  parameters
      integer Mm
      double precision x0,beta,d(-FMAXM:FMAXM)
c  local varibles
      integer m
      double precision C,I0

      d(0)=2*x0*I0(beta)
      do m=1,Mm      
        d(m)=sin(2*pi*m*x0)/(pi*m)*I0(beta*sqrt(1-m*m/dble(Mm*Mm)))
        d(-m)=d(m)
      enddo
      C=0
      do m=-Mm,Mm
        C=C+d(m)
      enddo
      C=1/C
      do m=-Mm,Mm
        d(m)=C*d(m)
      enddo
      return
      end

c****************************************************************
       double precision function filter_filter(d,w,Mm)
c****************************************************************
c
c  filter the data w with filter d
c  d    filter
c  Mn   dimension of the filter and data
c  w    input data
c
      implicit none
      include 'filter.inc'
      double precision d(-FMAXM:FMAXM),w(0:FMAXN-1)
      integer Mm

      integer m
      double precision w2

      w2=0.0d0
      do m=-Mm,Mm
        w2=w2+d(m)*w(Mm-m)
      enddo
      filter_filter=w2
      return
      end

