c**********************************************************************
c IO_INIT_TH.F
c**********************************************************************
c Read in thermal data, precompute thermal parameters for getacch subroutine
c
c Input:
c  infile	name of the thermal data input file
c  ntp		number of test particles
c
c Output:
c  dtreorient		reorientation time step
c  tau_reor(NTPMAX)	mean reorientation times
c  tau_disr(NTPMAX)	mean disruption times
c  iseed		number to initialize ran1()
c  iflgchk		if bit 6 is set, then write reorient.out file
c
c  in common block /yarko/
c  s(3,NTPMAX)		orientation of the spin axis	
c  lambda0(NTPMAX)	precomputed thermal parameters
c  Tstar40(NTPMAX)	
c  rho0(NTPMAX)		
c  psi(NTPMAX)		precomputed complex parameter
c  k_seasonal		number of iterations
c
c Remarks: 
c Authors:  Miroslav Broz, miroslav.broz@usa.net
c Date:  7/14/1998 

	subroutine io_init_th(infile,ntp,dtreorient,
     &    tau_reor,tau_disr,iseed,iflgchk)

	include 'swift.inc'

c...    Input
	character*(*) infile
        integer ntp,iflgchk

c...    Output
        real*8 dtreorient,tau_reor(NTPMAX),tau_disr(NTPMAX)
        integer iseed

        real*8 k_gauss
        parameter(k_gauss=0.01720209895d0)

c  thermal variables are given to getacch subroutine in common block /yarko/
      integer MAXk_seasonal             ! max # of iterations in calculation
      parameter(MAXk_seasonal=7)        ! of seasonal yarkovski effect
      integer*2 k_seasonal
      real*8 s(3,NTPMAX),lambda0(NTPMAX),Tstar40(NTPMAX),rho0(NTPMAX),
     :  lambdas0(NTPMAX),Rdash0(NTPMAX),thetas(NTPMAX),rhos0(NTPMAX),
     :  koefsin(MAXk_seasonal,NTPMAX),koefcos(MAXk_seasonal,NTPMAX)
      complex*8 psi(NTPMAX)
      common/yarko/s,psi,lambda0,Tstar40,rho0,lambdas0,Rdash0,
     :  thetas,rhos0,koefsin,koefcos,k_seasonal

c...   Internal
        integer i,ierr,ntpchk
        real*8 sigma,cv,S0,theta
c  Boltzmann constant [J.K^(-4).m^(-2).s^(-1)]
c        parameter(sigma=5.6703d-8)
        parameter(sigma=5.66962d-8)
c  speed of light [m.s^(-1)]
        parameter(cv=299792458.0d0)
c  solar constant [W.m^(-2)]
c        parameter(S0=1370)
        parameter(S0=1371)

        real*8 R,rho,rho_surface,K,C,A,epsilon,T,s1,s2
        real*8 mass,Gama,ls,Rdash,omega
        real*8 radius(NTPMAX),tau_reor0,tau_disr0
        complex*8 sinz,cosz,z
        logical write_reorient,write_disrupt

      common /debug/ radius

c-----
c...  Executable code      

      write(*,*) 'Thermal data file called ',infile
      call io_open(7,infile,'old','formatted',ierr)

      read(7,*) ntpchk
      if(ntp.eq.0) ntp = ntpchk
      if(ntpchk.ne.ntp) then
        write(*,*) ' SWIFT ERROR: in io_init_th'
        write(*,*) '     Number of test particles differs in ',infile
        call util_exit(1)
      endif
 
      do i=1,ntp
        read(7,*) R,rho,rho_surface,K,C,A,epsilon,T,s1,s2
        s1=s1/DEGRAD
        s2=s2/DEGRAD
c
c  precompute parameters for getacch subroutine
c  calculation of omega, if not given
c  omega = 1/R, 1 km sized asteroid has the rotational period 5 h
c
        if (T.le.0) then
          omega = TWOPI/(5*3600.)/(2*R/1000)
        else
          omega = TWOPI/(T*3600.)
        endif
c        write(*,*) 'omega = ',omega
        mass=4./3.*pi*R**3*rho
        Gama=sqrt(rho_surface*C*K)
        ls=sqrt(K/(rho_surface*C*omega))
        Rdash=R/ls
        Rdash0(i)=R/sqrt(K/(rho_surface*C))
        z=1/sqrt(2.)*(1.,-1.)*Rdash
c  canceled factor exp(-imag(z)) in fraction psi(i) to avoid numerical problems
        sinz=(cos(dble(z))+(0.,1.)*sin(dble(z))
     &    -(cos(-dble(z))+(0.,1.)*sin(-dble(z)))*exp(2.*imag(z)))
     &    /(0.,2.)
        cosz=(cos(dble(z))+(0.,1.)*sin(dble(z))
     &    +(cos(-dble(z))+(0.,1.)*sin(-dble(z)))*exp(2.*imag(z)))
     &    /2.
        psi(i)=((z**2-3.)*sinz+3.*z*cosz)/(sinz-z*cosz)
        s(1,i)=cos(s1)*cos(s2)
        s(2,i)=sin(s1)*cos(s2)
        s(3,i)=sin(s2)
c        write(*,*) 'omega =',omega,' s:',s(1,i),s(2,i),s(3,i)
        lambda0(i)=Gama*sqrt(omega)/(epsilon*sigma*sqrt(2.)*Rdash)
        theta = sqrt(2.)*Rdash*lambda0(i)
        lambdas0(i)=Gama/(epsilon*sigma*sqrt(2.))
        Tstar40(i)=(1.-A)*S0/(epsilon*sigma)
        rho0(i)=-4./9.*epsilon*sigma*pi*R**2/(mass*cv)
     &    *86400.**2/149597870e3
        radius(i)=R
        thetas(i) = Gama/(epsilon*sigma*Tstar40(i)**(3./4.))/
     &    sqrt(86400.0/k_gauss)
c debug code
c        write(*,*) 'K = ',K,' ls = ',
c     &    sqrt(K/rho_surface/C*86400.0/k_gauss),
c     &    'Thetas = ',Gama/(epsilon*sigma*Tstar40(i)**(3./4.))/
c     &    sqrt(86400.0/k_gauss)
        rhos0(i)=pi*(R**2)*S0/(mass*cv)*(1.-A)
      enddo

      read(7,*) k_seasonal
      if(ntpchk.ne.ntp) then
        write(*,*) ' SWIFT ERROR: in io_init_th'
        write(*,*) '     Number of iterations k_seasonal higher than ',
     :    MAXk_seasonal
        call util_exit(1)
      endif

      read(7,*) iseed
      read(7,*) dtreorient
      read(7,*) tau_reor0
      read(7,*) tau_disr0
      read(7,*) write_reorient
      read(7,*) write_disrupt

c  dtreorient, tau_reor0, tau_disr0 are given in yr
      dtreorient=dtreorient*365.25d0
c
c  expecting dependence of reorientation and disruption times tau_reor
c  on radius R in form sqrt(R)
c
      do i=1,ntp
        tau_reor(i)=365.25d0*tau_reor0*sqrt(radius(i))
        tau_disr(i)=365.25d0*tau_disr0*sqrt(radius(i))
      enddo

      if (write_reorient) iflgchk=ibset(iflgchk,6)
      if (write_disrupt) iflgchk=ibset(iflgchk,7)

      close(unit = 7)
      write(*,*) ' '

      return
      end    ! io_init_th.f
c-----------------------------------------------------------------

