c*************************************************************************
c IO_WRITE_FILTER
c*************************************************************************
c accumulate array of orbital elements for test particles and planets,
c filter/decimate it and write out a whole frame to an real*4 binary file.
c
c             Input:
c                 time          ==>  current time (real scalar)
c                 nbod          ==>  number of massive bodies (int scalar)
c                 ntp           ==>  number of test particles (int scalar)
c                 mass          ==>  mass of bodies (real array)
c                 xh,yh,zh      ==>  current position in helio coord 
c                                    (real arrays)
c                 vxh,vyh,vzh   ==>  current velocity in helio coord 
c                                    (real arrays)
c                 xht,yht,zht    ==>  current part position in helio coord 
c                                      (real arrays)
c                 vxht,vyht,vzht ==>  current velocity in helio coord 
c                                        (real arrays)
c                 istat           ==>  status of the test paricles
c                                      (2d integer array)
c                                      istat(i,1) = 0 ==> active:  = 1 not
c                                      istat(i,2) = -1 ==> Danby did not work
c                 oname           ==> output file name (character string) 
c                 iu              ==> unit number to write to
c                 fopenstat       ==>  The status flag for the open 
c                                      statements of the output files.  
c                                          (character*80)
c
c
c Remarks: Based on io_write_frame
c Authors: Miroslav Broz
c Date:    Dec 5th 1998
c Last revision: 

      subroutine io_write_filter(time,nbod,ntp,mass,xh,yh,zh,vxh,
     &           vyh,vzh,xht,yht,zht,vxht,vyht,vzht,istat,oname,
     &           iu,fopenstat)

      include 'swift.inc'
      include 'filter.inc'

c...  Inputs: 
      integer nbod,ntp,iu
      real*8 mass(nbod),time
      integer istat(NTPMAX,NSTAT)
      real*8 xh(nbod),yh(nbod),zh(nbod)
      real*8 vxh(nbod),vyh(nbod),vzh(nbod)
      real*8 xht(ntp),yht(ntp),zht(ntp)
      real*8 vxht(ntp),vyht(ntp),vzht(ntp)
      character*80 oname,fopenstat

c...  Internals
      integer i,n,id,k
      integer ialpha,ierr,FNskip
      real*8 gm
      integer i1st,i2nd	! =0 first time through; =1 after
      integer FN(FMAXC)	! index in elmts() buffer, (1;FBUFN)
      save i1st,i2nd,FN
c
c  parameters of filtration process are given in common block /filter/
c
      real*8 A(-FMAXM:FMAXM),B(-FMAXM:FMAXM),dtfilter
      integer filter_M,filter_N,filter_D(FMAXC),FBUFN
      character*10 filter_stat	! have to be the last in /filter/

      common/filter/A,B,dtfilter,filter_M,filter_N,filter_D,
     &  FBUFN,filter_stat
c
c  orbital elements are stored in common block /elements/, will be
c  used for computation of seasonal yarkovski force in getacc_tp.f
c 
      real*8 elmts(FMAXN,6,-FMAXNPL:FMAXNTP,FMAXC)
      real*8 t0
      integer index
      equivalence (index,FN(1))
      common/elements/elmts,t0,index

      data i1st,i2nd/0,0/

c----
c...  Executable code 

c  first time through
      if (i1st.eq.0) then

c  do this block only once
        if (i2nd.eq.0) then
          call io_open(iu,oname,fopenstat,'UNFORMATTED',ierr)
          if(ierr.ne.0) then
            write(*,*) ' SWIFT ERROR: in io_write_frame: '
            write(*,*) '     Could not open binary output file:'
            call util_exit(1)
          endif
          close(iu)

          do i=1,filter_N
            FN(i)=0
          enddo
c
c  skip first data due to alignment
c  how many points should we skip?
c
          k=filter_D(1)
          do i=2,filter_N
            k=k*filter_D(i)
          enddo
          n=2*filter_M
          do i=filter_N-1,1,-1
            n=filter_D(i)*n+2*filter_M
          enddo 
          n=n+1
          FNskip = n-(n/k+1)*k
          i2nd=1
        endif

c  wait until we can skip back to the beginning of the buffer
c  hopefully FNskip < buffer size
        if (FN(1).eq.FNskip) then
          FN(1)=0
          i1st=1
        endif
      endif

c  calculate orbital elements for planets and test particles
      t0 = time	! remember the epoch of mean anomaly
      FN(1)=FN(1)+1
      gm=mass(1)
      do i=1,ntp
        id=i
        call orbel_xv2el(xht(i),yht(i),zht(i),vxht(i),vyht(i),vzht(i),
     &    gm,ialpha,
     &    elmts(FN(1),1,id,1),elmts(FN(1),2,id,1),elmts(FN(1),3,id,1),
     &    elmts(FN(1),4,id,1),elmts(FN(1),5,id,1),elmts(FN(1),6,id,1))
      enddo

      do i=2,nbod
        gm=mass(1)+mass(i)
        id=-1*i
        call orbel_xv2el(xh(i),yh(i),zh(i),vxh(i),vyh(i),vzh(i),
     &    gm,ialpha,
     &    elmts(FN(1),1,id,1),elmts(FN(1),2,id,1),elmts(FN(1),3,id,1),
     &    elmts(FN(1),4,id,1),elmts(FN(1),5,id,1),elmts(FN(1),6,id,1))
      enddo
c
c  test if buffers are filled up
c
      do n=1,filter_N-1
        if (FN(n).ge.FBUFN) then

          FN(n+1)=FN(n+1)+1

c  filter the array
          if (filter_stat(n:n).eq.'A') then
            call filter_elmts(elmts,nbod,ntp,FN,n,filter_M,A)
          else	! if (filter_stat(n:n).eq.'B') then
            call filter_elmts(elmts,nbod,ntp,FN,n,filter_M,B)
          endif

c  decime with factor filter_D
          FN(n)=FN(n)-filter_D(n)

c  shift left
          call filter_shift(elmts,nbod,ntp,n,FBUFN,filter_D)
        endif
      enddo
c
c  test if the last buffer is filled up and write filtered elmts
c
      n=filter_N
      if (FN(n).ge.FBUFN) then
        call io_open(iu,oname,'append','UNFORMATTED',ierr)
        call io_write_hdr_r(iu,time,nbod,ntp,istat)
        if (filter_stat(n:n).eq.'A') then
          call filter_elmts_write(iu,elmts,n,nbod,ntp,istat,
     &      A,filter_M)
        else	! if (filter_stat(n:n).eq.'B') then
          call filter_elmts_write(iu,elmts,n,nbod,ntp,istat,
     &      B,filter_M)
        endif

c  decime with factor filter_D
        FN(n)=FN(n)-filter_D(n)

c  shift left
        call filter_shift(elmts,nbod,ntp,n,FBUFN,filter_D)
        close(iu)
      endif

      return
      end	! io_write_filter
c----------------------------------------------------------------------

c****************************************************************
      subroutine filter_elmts(elmts,nbod,ntp,FN,n,filter_M,d)
c****************************************************************
c
c  filter the array elmts with filter d and store the filtered
c  orbital elements in the level n+1 in array elmts
c
      implicit none
      include 'filter.inc'

      real*8 elmts(FMAXN,6,-FMAXNPL:FMAXNTP,FMAXC)
      integer nbod,ntp,n,FN(FMAXC),filter_M
      real*8 d(-FMAXM:FMAXM)

c  functions
      real*8 filter_filter

      integer i,k,id

      do i=2,nbod
        id=-1*i
        do k=1,6
          elmts(FN(n+1),k,id,n+1)=
     &      filter_filter(d,elmts(1,k,id,n),filter_M)
        enddo
      enddo

      do i=1,ntp
        id=i
        do k=1,6
          elmts(FN(n+1),k,id,n+1)=
     &      filter_filter(d,elmts(1,k,id,n),filter_M)
        enddo
      enddo

      return
      end	! filter_elmts

c****************************************************************
      subroutine filter_elmts_write(iu,elmts,n,nbod,ntp,istat,d,FM)
c****************************************************************
c
c  filter the array elmts with filter d and write filtered
c  orbital elements to the binary output file iu
c
      include 'swift.inc'
      include 'filter.inc'

      integer iu,nbod,ntp,n,FM,istat(NTPMAX,NSTAT)
      real*8 elmts(FMAXN,6,-FMAXNPL:FMAXNTP,FMAXC)
      real*8 d(-FMAXM:FMAXM)

      integer i,id
c  functions
      real*8 filter_filter

      do i=2,nbod
        id=-1*i
        call io_write_line_r(iu,i,
     &    filter_filter(d,elmts(1,1,id,n),FM),
     &    filter_filter(d,elmts(1,2,id,n),FM),
     &    filter_filter(d,elmts(1,3,id,n),FM),
     &    filter_filter(d,elmts(1,4,id,n),FM),
     &    filter_filter(d,elmts(1,5,id,n),FM),
     &    filter_filter(d,elmts(1,6,id,n),FM))
      enddo

      do i=1,ntp
        if(istat(i,1).eq.0) then
          id=i
          call io_write_line_r(iu,i,
     &      filter_filter(d,elmts(1,1,id,n),FM),
     &      filter_filter(d,elmts(1,2,id,n),FM),
     &      filter_filter(d,elmts(1,3,id,n),FM),
     &      filter_filter(d,elmts(1,4,id,n),FM),
     &      filter_filter(d,elmts(1,5,id,n),FM),
     &      filter_filter(d,elmts(1,6,id,n),FM))
        endif
      enddo

      return
      end	! filter_elmts_write

c****************************************************************
      subroutine filter_shift(elmts,nbod,ntp,n,FBUFN,filter_D)
c****************************************************************
c
c  shift left elements in array elmts by filter_D(n) / decimation
c
      implicit none
      include 'filter.inc'

      real*8 elmts(FMAXN,6,-FMAXNPL:FMAXNTP,FMAXC)
      integer nbod,ntp,n,FBUFN,filter_D(FMAXC)

      integer i,j,k,id

      do i=2,nbod
        id=-1*i
        do k=1,6
          do j=filter_D(n)+1,FBUFN
            elmts(j-filter_D(n),k,id,n)=elmts(j,k,id,n)
          enddo
        enddo
      enddo

      do i=1,ntp
        id=i
        do k=1,6
          do j=filter_D(n)+1,FBUFN
            elmts(j-filter_D(n),k,id,n)=elmts(j,k,id,n)
          enddo
        enddo
      enddo

      return
      end	! filter_shift

